# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime

from .._models import BaseModel
from .cloud_zone import CloudZone

__all__ = ["ScheduleOption"]


class ScheduleOption(BaseModel):
    co2_intensity: float

    time: datetime

    zone: CloudZone



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .._models import BaseModel
from .cloud_zone import CloudZone

__all__ = ["RegionListResponse"]


class RegionListResponse(BaseModel):
    regions: List[CloudZone]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable, Optional
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo
from .cloud_zone_param import CloudZoneParam

__all__ = ["ScheduleCreateParams", "Window"]


class ScheduleCreateParams(TypedDict, total=False):
    duration: Required[str]

    windows: Required[Iterable[Window]]

    zones: Required[Iterable[CloudZoneParam]]

    num_options: Optional[int]


class Window(TypedDict, total=False):
    end: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]

    start: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .._models import BaseModel
from .schedule_option import ScheduleOption

__all__ = ["ScheduleCreateResponse", "CarbonSavings"]


class CarbonSavings(BaseModel):
    vs_median_case: float

    vs_naive_case: float

    vs_worst_case: float


class ScheduleCreateResponse(BaseModel):
    carbon_savings: CarbonSavings

    ideal: ScheduleOption

    median_case: ScheduleOption

    naive_case: ScheduleOption

    options: List[ScheduleOption]

    worst_case: ScheduleOption



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["CloudZoneParam"]


class CloudZoneParam(TypedDict, total=False):
    provider: Required[Literal["aws", "gcp", "azure"]]

    region: Required[
        Literal[
            "us-east-1",
            "us-west-1",
            "eu-central-1",
            "ap-southeast-2",
            "us-central1",
            "eastus",
            "eastus2",
            "southcentralus",
            "westus2",
            "westus3",
            "northeurope",
            "swedencentral",
            "uksouth",
            "westeurope",
            "centralus",
            "francecentral",
            "germanywestcentral",
            "italynorth",
            "norwayeast",
            "polandcentral",
            "eastus2euap",
            "eastusstg",
            "northcentralus",
            "westus",
            "centraluseuap",
            "westcentralus",
            "francesouth",
            "germanynorth",
            "norwaywest",
            "ukwest",
        ]
    ]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from carbonaware_scheduler import CarbonawareScheduler, AsyncCarbonawareScheduler
from carbonaware_scheduler.types import ScheduleCreateResponse
from carbonaware_scheduler._utils import parse_datetime

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestSchedule:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create(self, client: CarbonawareScheduler) -> None:
        schedule = client.schedule.create(
            duration="PT1H",
            windows=[
                {
                    "end": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "start": parse_datetime("2019-12-27T18:11:19.117Z"),
                }
            ],
            zones=[
                {
                    "provider": "aws",
                    "region": "us-east-1",
                }
            ],
        )
        assert_matches_type(ScheduleCreateResponse, schedule, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_create_with_all_params(self, client: CarbonawareScheduler) -> None:
        schedule = client.schedule.create(
            duration="PT1H",
            windows=[
                {
                    "end": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "start": parse_datetime("2019-12-27T18:11:19.117Z"),
                }
            ],
            zones=[
                {
                    "provider": "aws",
                    "region": "us-east-1",
                }
            ],
            num_options=0,
        )
        assert_matches_type(ScheduleCreateResponse, schedule, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_create(self, client: CarbonawareScheduler) -> None:
        response = client.schedule.with_raw_response.create(
            duration="PT1H",
            windows=[
                {
                    "end": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "start": parse_datetime("2019-12-27T18:11:19.117Z"),
                }
            ],
            zones=[
                {
                    "provider": "aws",
                    "region": "us-east-1",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        schedule = response.parse()
        assert_matches_type(ScheduleCreateResponse, schedule, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_create(self, client: CarbonawareScheduler) -> None:
        with client.schedule.with_streaming_response.create(
            duration="PT1H",
            windows=[
                {
                    "end": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "start": parse_datetime("2019-12-27T18:11:19.117Z"),
                }
            ],
            zones=[
                {
                    "provider": "aws",
                    "region": "us-east-1",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            schedule = response.parse()
            assert_matches_type(ScheduleCreateResponse, schedule, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncSchedule:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_create(self, async_client: AsyncCarbonawareScheduler) -> None:
        schedule = await async_client.schedule.create(
            duration="PT1H",
            windows=[
                {
                    "end": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "start": parse_datetime("2019-12-27T18:11:19.117Z"),
                }
            ],
            zones=[
                {
                    "provider": "aws",
                    "region": "us-east-1",
                }
            ],
        )
        assert_matches_type(ScheduleCreateResponse, schedule, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncCarbonawareScheduler) -> None:
        schedule = await async_client.schedule.create(
            duration="PT1H",
            windows=[
                {
                    "end": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "start": parse_datetime("2019-12-27T18:11:19.117Z"),
                }
            ],
            zones=[
                {
                    "provider": "aws",
                    "region": "us-east-1",
                }
            ],
            num_options=0,
        )
        assert_matches_type(ScheduleCreateResponse, schedule, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncCarbonawareScheduler) -> None:
        response = await async_client.schedule.with_raw_response.create(
            duration="PT1H",
            windows=[
                {
                    "end": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "start": parse_datetime("2019-12-27T18:11:19.117Z"),
                }
            ],
            zones=[
                {
                    "provider": "aws",
                    "region": "us-east-1",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        schedule = await response.parse()
        assert_matches_type(ScheduleCreateResponse, schedule, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncCarbonawareScheduler) -> None:
        async with async_client.schedule.with_streaming_response.create(
            duration="PT1H",
            windows=[
                {
                    "end": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "start": parse_datetime("2019-12-27T18:11:19.117Z"),
                }
            ],
            zones=[
                {
                    "provider": "aws",
                    "region": "us-east-1",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            schedule = await response.parse()
            assert_matches_type(ScheduleCreateResponse, schedule, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["CloudZone"]


class CloudZone(BaseModel):
    provider: Literal["aws", "gcp", "azure"]

    region: Literal[
        "us-east-1",
        "us-west-1",
        "eu-central-1",
        "ap-southeast-2",
        "us-central1",
        "eastus",
        "eastus2",
        "southcentralus",
        "westus2",
        "westus3",
        "northeurope",
        "swedencentral",
        "uksouth",
        "westeurope",
        "centralus",
        "francecentral",
        "germanywestcentral",
        "italynorth",
        "norwayeast",
        "polandcentral",
        "eastus2euap",
        "eastusstg",
        "northcentralus",
        "westus",
        "centraluseuap",
        "westcentralus",
        "francesouth",
        "germanynorth",
        "norwaywest",
        "ukwest",
    ]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .cloud_zone import CloudZone as CloudZone
from .schedule_option import ScheduleOption as ScheduleOption
from .cloud_zone_param import CloudZoneParam as CloudZoneParam
from .retrieve_response import RetrieveResponse as RetrieveResponse
from .region_list_response import RegionListResponse as RegionListResponse
from .health_check_response import HealthCheckResponse as HealthCheckResponse
from .schedule_create_params import ScheduleCreateParams as ScheduleCreateParams
from .schedule_create_response import ScheduleCreateResponse as ScheduleCreateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, Optional

import httpx

from ..types import schedule_create_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.cloud_zone_param import CloudZoneParam
from ..types.schedule_create_response import ScheduleCreateResponse

__all__ = ["ScheduleResource", "AsyncScheduleResource"]


class ScheduleResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ScheduleResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/carbon-aware/scheduler-client-python#accessing-raw-response-data-eg-headers
        """
        return ScheduleResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ScheduleResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/carbon-aware/scheduler-client-python#with_streaming_response
        """
        return ScheduleResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        duration: str,
        windows: Iterable[schedule_create_params.Window],
        zones: Iterable[CloudZoneParam],
        num_options: Optional[int] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ScheduleCreateResponse:
        """
        Schedule

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v0/schedule/",
            body=maybe_transform(
                {
                    "duration": duration,
                    "windows": windows,
                    "zones": zones,
                    "num_options": num_options,
                },
                schedule_create_params.ScheduleCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ScheduleCreateResponse,
        )


class AsyncScheduleResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncScheduleResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/carbon-aware/scheduler-client-python#accessing-raw-response-data-eg-headers
        """
        return AsyncScheduleResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncScheduleResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/carbon-aware/scheduler-client-python#with_streaming_response
        """
        return AsyncScheduleResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        duration: str,
        windows: Iterable[schedule_create_params.Window],
        zones: Iterable[CloudZoneParam],
        num_options: Optional[int] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ScheduleCreateResponse:
        """
        Schedule

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v0/schedule/",
            body=await async_maybe_transform(
                {
                    "duration": duration,
                    "windows": windows,
                    "zones": zones,
                    "num_options": num_options,
                },
                schedule_create_params.ScheduleCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ScheduleCreateResponse,
        )


class ScheduleResourceWithRawResponse:
    def __init__(self, schedule: ScheduleResource) -> None:
        self._schedule = schedule

        self.create = to_raw_response_wrapper(
            schedule.create,
        )


class AsyncScheduleResourceWithRawResponse:
    def __init__(self, schedule: AsyncScheduleResource) -> None:
        self._schedule = schedule

        self.create = async_to_raw_response_wrapper(
            schedule.create,
        )


class ScheduleResourceWithStreamingResponse:
    def __init__(self, schedule: ScheduleResource) -> None:
        self._schedule = schedule

        self.create = to_streamed_response_wrapper(
            schedule.create,
        )


class AsyncScheduleResourceWithStreamingResponse:
    def __init__(self, schedule: AsyncScheduleResource) -> None:
        self._schedule = schedule

        self.create = async_to_streamed_response_wrapper(
            schedule.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .health import (
    HealthResource,
    AsyncHealthResource,
    HealthResourceWithRawResponse,
    AsyncHealthResourceWithRawResponse,
    HealthResourceWithStreamingResponse,
    AsyncHealthResourceWithStreamingResponse,
)
from .regions import (
    RegionsResource,
    AsyncRegionsResource,
    RegionsResourceWithRawResponse,
    AsyncRegionsResourceWithRawResponse,
    RegionsResourceWithStreamingResponse,
    AsyncRegionsResourceWithStreamingResponse,
)
from .schedule import (
    ScheduleResource,
    AsyncScheduleResource,
    ScheduleResourceWithRawResponse,
    AsyncScheduleResourceWithRawResponse,
    ScheduleResourceWithStreamingResponse,
    AsyncScheduleResourceWithStreamingResponse,
)

__all__ = [
    "ScheduleResource",
    "AsyncScheduleResource",
    "ScheduleResourceWithRawResponse",
    "AsyncScheduleResourceWithRawResponse",
    "ScheduleResourceWithStreamingResponse",
    "AsyncScheduleResourceWithStreamingResponse",
    "RegionsResource",
    "AsyncRegionsResource",
    "RegionsResourceWithRawResponse",
    "AsyncRegionsResourceWithRawResponse",
    "RegionsResourceWithStreamingResponse",
    "AsyncRegionsResourceWithStreamingResponse",
    "HealthResource",
    "AsyncHealthResource",
    "HealthResourceWithRawResponse",
    "AsyncHealthResourceWithRawResponse",
    "HealthResourceWithStreamingResponse",
    "AsyncHealthResourceWithStreamingResponse",
]
